@@ -67,14 +67,17 @@ if (flag)
 	return AsInt(1);
 	else {
 
+#ifndef MAC
 #ifdef OPENMPI
     if (!dlopen("libmpi.so.0", RTLD_GLOBAL | RTLD_LAZY) 
-	&& !dlopen("libmpi.so", RTLD_GLOBAL | RTLD_LAZY) 
-	&& !dlopen("libmpi.dylib", RTLD_GLOBAL | RTLD_LAZY)) {
-        Rprintf("%s\n",dlerror());
+	&& !dlopen("libmpi.so", RTLD_GLOBAL | RTLD_LAZY)){
+	//&& !dlopen("libmpi.dylib", RTLD_GLOBAL | RTLD_LAZY)
+ 	//&& !dlopen("libmpi.1.dylib", RTLD_GLOBAL | RTLD_LAZY)) {
+       Rprintf("%s\n",dlerror());
         return AsInt(0);
     }
 #endif
+#endif
 
 #ifndef MPI2
    	fake_argv[0] = (char *)&fake_argv0;
